from .extractors import eztv, yts
import os
from halo import Halo


class Ezflix(object):
    def __init__(self,
                 query,
                 media_type='tv',
                 limit=20,
                 sort_by='seeds',
                 sort_order='desc',
                 quality=None,
                 minimum_rating=None,
                 language='en'
                 ):
        self._torrents = []
        self._query = query
        self._media_type = media_type
        self._limit = limit
        self._sort_by = sort_by
        self._sort_order = sort_order
        self._quality = quality
        self._minimum_rating = minimum_rating
        self._language = language

    def get_magnet(self, val):
        for result in self._torrents:
            if result['id'] == int(val):
                return result
        return None

    def find_subtitles(self, title):
        os.system("subliminal download -l %s '%s'" % (self._language, title))
        cur_dir = os.getcwd()
        file_list = os.listdir(cur_dir)
        for f in file_list:
            if title in f:
                return f

        return None

    def get_torrents(self):
        spinner = Halo(text='Searching...', spinner='dots')
        spinner.start()
        if self._media_type == 'tv':
            self._torrents = eztv(self._query.replace(' ', '-').lower(), limit=self._limit, quality=self._quality)

        elif self._media_type == 'movie':
            self._torrents = yts(query_term=self._query,
                                 limit=self._limit,
                                 sort_by=self._sort_by,
                                 sort_order=self._sort_order,
                                 quality=self._quality,
                                 minimum_rating=self._minimum_rating
                                 )
        spinner.stop()
        spinner.clear()
        return self._torrents



from .utils import cmd_exists, peerflix
import sys
import colorful
from .argument_parser import Parser
from .ezflix import Ezflix
from prettytable import PrettyTable
from builtins import input
import logging

logging.basicConfig()
logger = logging.getLogger(__file__)

parser = Parser()
args = parser.parse()

media_player = args.media_player

if not cmd_exists('peerflix'):
    sys.exit('This program requires Peerflix. https://github.com/mafintosh/peerflix')

if not cmd_exists('mpv') and args.media_player == 'mpv':
    logger.warning("In ezflix, the default player is mpv. It will fallback to vlc if mpv isn't found. You can use the media_player argument to set your media player.")
    media_player = 'vlc'

if not (args.query and args.query.strip()): 
    sys.exit(colorful.red("Search query not valid."))


def main():
    ezflix = Ezflix(query=args.query,
                    media_type=args.media_type,
                    limit=args.limit,
                    sort_by=args.sort_by,
                    sort_order=args.sort_order,
                    quality=args.quality,
                    minimum_rating=args.minimum_rating,
                    language=args.language
                    )

    torrents = ezflix.get_torrents()
    if torrents is None or len(torrents) == 0:
        sys.exit(colorful.red('No results found.'))
    if args.latest:
        latest = torrents[0]
        file_path = ''
        if args.subtitles:
            file_path = ezflix.find_subtitles(latest['title'])
        print("Playing " + latest['title'])
        peerflix(latest['magnet'], media_player, args.media_type, args.subtitles, args.remove, file_path)
        sys.exit()
    row = PrettyTable()
    row.field_names = ["Id", "Torrent", "Seeds", "Peers"]
    row.align = 'l'
    for result in torrents:
        row.add_row([result['id'], result['title'], result['seeds'], result['peers']])
    print(row)
    print(colorful.bold("Make selection: (Enter quit to close the program)"))
    while True:
        read = input()
        if read == 'quit':
            sys.exit()
        try:
            int_val = int(read)
        except (ValueError, TypeError) as error:
            print(colorful.red('Invalid selection.'))
            continue
        magnet = ezflix.get_magnet(int_val)
        if not magnet:
            print(colorful.red('Invalid selection.'))
            continue
        print("Playing " + magnet['title'])
        file_path = ''
        if args.subtitles:
            file_path = ezflix.find_subtitles(magnet['title'])
        peerflix(magnet['magnet'], media_player, args.media_type, args.subtitles, args.remove, file_path)


if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        sys.exit()



import requests
from bs4 import BeautifulSoup
import sys


def eztv(q, limit, quality=None):
    limit = int(limit)
    url = 'https://eztv.ag/search/' + q
    req = requests.get(url)

    if not req.ok:
        return

    soup = BeautifulSoup(req.text, 'html.parser')
    magnets = soup.find_all('a', {'class': 'magnet'}, href=True)

    if magnets is None:
        sys.exit('No results found')

    arr, count = [], 1
    for magnet in magnets:

        if count == limit + 1:
            break

        if q.lower().strip()[0] in magnet['title'].lower():
            seeds = None
            try:
                seeds = magnet.find_parent().find_parent().find("font").get_text() # verified for the edge cases
            except AttributeError as e:
                pass
            peers = "-" # as eztv doesn't give any peers detail, atleast not on the search page.
            if quality is not None and quality in magnet['title']:
                arr.append({'id': count, 'title': magnet['title'][:-12], 'magnet': magnet['href'], 'seeds': seeds, 'peers': peers})
                count += 1
            else:
                arr.append({'id': count, 'title': magnet['title'][:-12], 'magnet': magnet['href'], 'seeds': seeds, 'peers': peers})
                count += 1
    return arr



from argparse import ArgumentParser


class Parser(object):
    def __init__(self):
        self._parser = ArgumentParser(prog='ezflix')
        self._supported_players = [
            'mpv',
            'vlc',
            'mplayer',
            'smplayer',
            'mpchc',
            'potplayer',
            'webplay',
            'omx',
            'airplay'
        ]

        self._sort_types = [
            'download_count',
            'like_count',
            'date_added',
            'seeds',
            'peers',
            'rating',
            'title',
            'year'
        ]

        self._sort_orders = [
            'asc',
            'desc'
        ]

        self._media_types = [
            'movie',
            'tv'
        ]

        self._qualities = [
            '720p',
            '1080p',
            '3d'
        ]
        self._add_arguments()

    def _add_arguments(self):
        self._parser.add_argument(
            'media_type',
            help="The media type.",
            default='tv',
            nargs='?',
            choices=self._media_types
        )
        self._parser.add_argument(
            'query',
            help="The search query."
        )
        self._parser.add_argument(
            '--limit',
            help="The number of results to return.",
            default='20',
            nargs='?'
        )
        self._parser.add_argument(
            '--minimum_rating',
            help="Used to filter movie by a given minimum IMDb rating",
            nargs='?'
        )
        self._parser.add_argument(
            '--media_player',
            help="The media player.",
            default='mpv',
            nargs='?',
            choices=self._supported_players
        )
        self._parser.add_argument(
            '--latest',
            help="Play the latest TV episode.",
            dest='latest',
            action='store_true'
        )
        self._parser.add_argument(
            '--subtitles',
            help="Load subtitles file.",
            dest='subtitles',
            action='store_true')
        self._parser.add_argument(
            '--sort_by',
            help="Use this argument to sort the torrents.",
            default='seeds',
            nargs='?',
            choices=self._sort_types
        )
        self._parser.add_argument(
            '--sort_order',
            help="Use this argument to set the sort order.",
            default='desc',
            nargs='?',
            choices=self._sort_orders
        )
        self._parser.add_argument(
            '--quality',
            help="Use this argument to set the min quality.",
            nargs='?',
            choices=self._qualities
        )
        self._parser.add_argument(
            '--genre',
            help='Used to filter by a given genre (See http://www.imdb.com/genre/ for full list)'
        )

        self._parser.add_argument(
            '--remove',
            help='Remove files on exit.',
            dest='remove',
            action='store_true'
        )

        self._parser.add_argument(
            '--language',
            help='Language as IETF code. Set this argument to download subtitles in a given language.',
            default='en'
        )

    def parse(self):
        return self._parser.parse_args()



# -*- coding: utf-8 -*-

import os
import sys
from shutil import rmtree

from setuptools import setup, Command

with open("README.rst", "rb") as f:
    long_descr = f.read().decode("utf-8")

HERE = os.path.abspath(os.path.dirname(__file__))

version_ns = {}

with open(os.path.join(HERE, 'ezflix', '__version__.py')) as f:
    exec(f.read(), {}, version_ns)
    

class PublishCommand(Command):
    """Support setup.py publish."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(HERE, 'dist'))
        except:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload dist/*')

        sys.exit()


setup(
    name="ezflix",
    packages=["ezflix", "ezflix.extractors"],
    entry_points={
        "console_scripts": ['ezflix = ezflix.main:main']
    },
    version=version_ns['__version__'],
    keywords=['torrents', 'streaming', 'movies', 'tv', 'yify', 'eztv', 'peerflix'],
    description="Command line utility to search for TV and movie torrents and stream using Peerflix automatically.",
    long_description=long_descr,
    author="Anthony Bloomer",
    author_email="ant0@protonmail.ch",
    url="https://github.com/AnthonyBloomer/ezflix",
    install_requires=[
        'beautifulsoup4',
        'requests',
        'colorful',
        'subliminal',
        'halo',
        'PTable',
        'future'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    cmdclass={
        'publish': PublishCommand,
    }
)



import requests


def yts(query_term, quality=None, limit=20, minimum_rating=4, sort_by='seeds', sort_order='desc'):
    params = {
        'query_term': query_term,
        'sort_by': sort_by,
        'sort_order': sort_order,
        'limit': limit,
        'quality': quality,
        'minimum_rating': minimum_rating

    }
    req = requests.get('https://yts.ag/api/v2/list_movies.json', params=params)

    if not req.ok:
        return

    req = req.json()

    if not req['status'] == 'ok' or not req['data']['movie_count'] > 0:
        return

    arr, count = [], 1
    for r in req['data']['movies']:
        if 'torrents' in r:
            for torrent in r['torrents']:
                title = '%s (%s) (%s)' % (r['title'], r['year'], torrent['quality'])
                if quality is not None and quality == torrent['quality']:
                        arr.append({'id': count, 'title': title, 'magnet': torrent['url'], 'seeds': torrent['seeds'], 'peers': torrent['peers']})
                        count += 1
                else:
                    arr.append({'id': count, 'title': title, 'magnet': torrent['url'], 'seeds': torrent['seeds'], 'peers': torrent['peers']})
                    count += 1
    return arr
